#include "TimePixTOTDistributionProcessor.h"
#include "TimePixMapHandlerProcessor.h"
#include "TimePixPixelMode.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
//#include <EVENT/Track.h>
#include <EVENT/TrackerData.h>
//#include "IntersectionCalculator.h"

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
// endif

#include <marlin/Global.h>

#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"

using namespace lcio ;
using namespace marlin ;
using namespace tpcconddata;

namespace marlintpc
{

TimePixTOTDistributionProcessor aTimePixTOTDistributionProcessor;
    
TimePixTOTDistributionProcessor::TimePixTOTDistributionProcessor(): Processor("TimePixTOTDistributionProcessor")
						    , _distributionTOTHisto(0) //,
						      //_distributionYHisto(0), 
						      //_distributionZHisto(0), 
						      //_distributionXYHisto(0)
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERDATA,
				"InputData" , 
				"Name of the input data collection"  ,
				_inputColName ,
				std::string("TimePixInterpolData") ) ;
    // modify processor description
    _description = "TimePixTOTDistributionProcessor: Fill x, y, and z distribution of all hits into AIDA histgram" ;

  registerProcessorParameter( "HistogramMinValueTOT" , 
			      "Minimum value in x histogram (default: 0. mm)"  ,
			      _histogramMinValueTOT ,
			      float(0.) ) ;

  registerProcessorParameter( "HistogramMaxValueTOT" , 
			      "Maximal value in x histogram (default:  15. mm)"  ,
			      _histogramMaxValueTOT ,
			      float(1000.) ) ;

//  registerProcessorParameter( "HistogramMinValueY" , 
//			      "Minimum value in y histogram (default: 0. mm)"  ,
//			      _histogramMinValueY ,
//			      float(0.) ) ;
//
//  registerProcessorParameter( "HistogramMaxValueY" , 
//			      "Maximal value in y histogram (default:  15. mm)"  ,
//			      _histogramMaxValueY ,
//			      float(15.) ) ;
//
//  registerProcessorParameter( "HistogramMinValueZ" , 
//			      "Minimum value in z histogram (default: 0.0 mm)"  ,
//			      _histogramMinValueZ ,
//			      float(0.0) ) ;
//
//  registerProcessorParameter( "HistogramMaxValueZ" , 
//			      "Maximal value in z histogram (default:  260.0 mm)"  ,
//			      _histogramMaxValueZ ,
//			      float(260.0) ) ;
}

TimePixTOTDistributionProcessor::~TimePixTOTDistributionProcessor()
{
}

void TimePixTOTDistributionProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;
	
	int TOTbins = static_cast<int>(_histogramMaxValueTOT - _histogramMinValueTOT);

	// get a histograms from the AIDAProcessor
	_distributionTOTHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "TimePixTOTDistributionHisto", 
			     "TimePixTOTDistributionHisto", 
			     TOTbins, _histogramMinValueTOT, _histogramMaxValueTOT);

//	_distributionYHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "yDistributionHisto", 
//			     "yDistributionHisto", 
//			     100, _histogramMinValueY, _histogramMaxValueY);
//
//	_distributionZHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "zDistributionHisto", 
//			     "zDistributionHisto", 
//			     100, _histogramMinValueZ, _histogramMaxValueZ);
//	_distributionXYHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram2D( "xyDistributionHisto", 
//			     "xyDistributionHisto", 
//			     900, _histogramMinValueX, _histogramMaxValueX,
//			     900, _histogramMinValueY, _histogramMaxValueY);



    }

void TimePixTOTDistributionProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
    _timePixMap = TimePixMapHandlerProcessor::getTimePixMap();
	
}
    
void TimePixTOTDistributionProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    streamlog_out(MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;


    // we need a LCCollectionVec to use iterators, as LCCollectio is not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	streamlog_out(DEBUG0)<<"no imput data"<<std::endl;
	// no input data
	// just continue ...
	return;
    }


    // cast did not succeed
    if( InputCollection == 0 )
    {
	streamlog_out(ERROR2) << "InputCollection vector is 0" << std::endl;
	return;
    }

    for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	it < InputCollection->end(); it ++ )
    {
	streamlog_out(DEBUG0)<<"casting data"<<std::endl;
	TrackerData* data = dynamic_cast<TrackerData *>(*it);
	
	if ( data == 0 ){
	  streamlog_out(DEBUG0)<<"cast failed"<<std::endl;
	}
	else {
	  streamlog_out(DEBUG0)<<"cast succesfull"<<std::endl;
	}

	if ( data == 0 ) break;
	else {
	    
	    

	    int pixelID = data->getCellID0();
	    int chipID  = data->getCellID1();

	    streamlog_out(DEBUG0)<<"pixelID : "<<pixelID<<"  chipId "<<chipID<<std::endl;

	    FloatVec ChargeVec = data->getChargeValues();
	    
	    for ( FloatVec::iterator ch = ChargeVec.begin(); ch < ChargeVec.end(); ch++ ) {
	    
		streamlog_out(DEBUG0)<<"ChargeVecIteration "<<pixelID<<"  "<<*ch<<std::endl;

	        streamlog_out(DEBUG0)<<"mode = "<<((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, pixelID ) ))
						   ->second)->getPixelMode() <<std::endl;

		if ( ((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, pixelID ) ))
		      ->second)->getPixelMode() == TimePixPixelMode::TOT ) 
		    _distributionTOTHisto->fill(*ch);
		
		pixelID++;
	    
	    }
	
	}

//	_distributionYHisto->fill(pos[1]);
//	_distributionZHisto->fill(pos[2]);
////	    if (pos[2] < 10)
//	_distributionTOTYHisto->fill(pos[0],pos[1]);
	
    }
    
}


}// namespace 
