#include "EFieldVector.h"

namespace marlintpc{

EFieldVector::EFieldVector(int xBin, int yBin, int zBin, float Ex , float Ey , float Ez) {

  setXBin( xBin );
  setYBin( yBin );
  setZBin( zBin );

  if( Ex ) setEx( Ex );
  if( Ey ) setEy( Ey );
  if( Ez ) setEz( Ez );
}

EFieldVector::~EFieldVector(){ }

int EFieldVector::getXBin() const {

    return getIntVal( 0 ) ;
}


int EFieldVector::getYBin() const {

    return getIntVal( 1 ) ;
}


int EFieldVector::getZBin() const {

    return getIntVal( 2 ) ;
}

float EFieldVector::getEx() const {

  return getFloatVal( 0 );
}

float EFieldVector::getEy() const {

  return getFloatVal( 1 );
}

float EFieldVector::getEz() const {

  return getFloatVal( 2 );
}


bool EFieldVector::operator==(const EFieldVector &right) const {

  return (  (this->getXBin() == right.getXBin()) 
	    && (this->getYBin() == right.getYBin()) 
               && (this->getZBin() == right.getZBin()) );
}
  
bool EFieldVector::operator<(const EFieldVector &right) const {

  // first test the xBins
  if (this->getXBin() < right.getXBin()) return true;
  if (this->getXBin() > right.getXBin()) return false;

  // xBins are equal, test yBins
  if (this->getYBin() < right.getYBin()) return true;
  if (this->getYBin() > right.getYBin()) return false;

  // yBins are equal, test zBins
  if (this->getZBin() < right.getZBin()) return true;
  // we can skip this as the return value would be false, which is returned anyhow
  //    if (this->getYBin() > right.getYBin()) return false;
  return false;
}

EFieldVector& EFieldVector::operator=(const EFieldVector& that)
{
  if (this == &that) return *this; // avoid fatal self-assignment

  if (_createdObject)
    delete _obj;

  _createdObject=that._createdObject;

  if (_createdObject)
    _obj = new  LCGenericObjectImpl(*(that._obj));
  else
    _obj = that._obj;

  return *this;
}

void EFieldVector::print ( std::ostream& os ) const {
    os<<"EFieldVector: xBin  "<< getXBin()
      <<" ; yBin "            << getYBin()
      <<" ; ZBin "            << getZBin()
      <<" Ex,Ey,Ez " << getEx() << " " 
                     << getEy() << " " 
                     << getEz() << std::endl;
} 

std::ostream &operator<<(std::ostream &os, const EFieldVector &efv) {
    efv.print(os);
    return os;
}

void EFieldVector::setXBin(int xBin){

    obj()->setIntVal( 0, xBin ) ;
}

void EFieldVector::setYBin(int yBin){

    obj()->setIntVal( 1, yBin ) ;
}


void EFieldVector::setZBin(int zBin){

    obj()->setIntVal( 2, zBin ) ;
}

 
void EFieldVector::setEx(float Ex){

  obj()->setFloatVal( 0, Ex);
}

void EFieldVector::setEy(float Ey){

  obj()->setFloatVal( 1, Ey );
}

void EFieldVector::setEz(float Ez){

  obj()->setFloatVal( 2 , Ez );
}


std::string EFieldVector::getRevision(){
    return std::string("$Rev:$");
}

}
