#ifdef USE_LCCD

#ifndef PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H
#define PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

// GEAR
#include <gear/TPCParameters.h>
#include <gearimpl/GlobalPadIndex.h>

// LCIO
#include <EVENT/TrackerHit.h>

/* NOTE this uses the C++ version of Minuit - not the Fortran version */
#include <Minuit2/FCNBase.h>

// CLHEP
#include <CLHEP/Vector/TwoVector.h>

// stl
#include <string>

namespace EVENT {

	class TrackerPulse;
}

namespace AIDA {

	class ITuple;
}

namespace marlintpc {

/** PhotoelectricLikelihoodFittingProcessor: performs a likelihood fit on photoelectric data.
 *    @author J. Abernathy (UVic)<br>
 */
class PhotoelectricLikelihoodFittingProcessor : public marlin::Processor {

public:

	PhotoelectricLikelihoodFittingProcessor() ;

	virtual void init() ;

	virtual Processor*  newProcessor() { return new PhotoelectricLikelihoodFittingProcessor(); }

	virtual void processRunHeader( lcio::LCRunHeader* run );

	virtual void processEvent( lcio::LCEvent * evt );

	virtual void check( lcio::LCEvent * evt );

	virtual void end();

private:

	/** the name of the input collection of TrackerHits which represent seed photodots
	 */
	std::string _input_seed_photodot_collection_name;

	/** the name of the input collection of Tracks which represent seed photolines
	 */
	std::string _input_seed_photoline_collection_name;

	/** the name of the output collection of TrackerHits which represent fit photodots
	 */
	std::string _output_photodot_collection_name;

	/** the name of the output collection of Tracks which represent fit photolines
	 */
	std::string _output_photoline_collection_name;

	/** determines whether the output of this processor will be stored in a file after program termination (false) or not (true - default)
	*/
	bool _output_is_transient;

	/** the expected diffusion of photodot charge due to the drift distance and initial defocussing
	 */
	double _sigma;

	/** this tuple holds the results of each photodot fit
	 */
	AIDA::ITuple *_photodot_fit_results_tuple;

	/** this helper class is used to calculate the likelihood function while fitting photodots
	 */
	class PhotodotLikelihood : public ROOT::Minuit2::FCNBase {

	public:

		PhotodotLikelihood ( const TrackerHit &photodot_data );

		virtual double operator()(const std::vector<double>& x) const;

		virtual double Up () const;

	private:

		const TrackerHit &_photodot_data;
	};

};

}

#endif // PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H
#endif // USE_LCCD
