#ifndef ADCElectronicsHandler_h
#define ADCElectronicsHandler_h 1

//C++
#include <iostream>
#include <utility>
#include <stdexcept>

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
#include "lccd/IConditionsChangeListener.hh"

//Marlin
#include "marlin/ConditionsProcessor.h"

//TPCCondData
#include "GenericADCElectronicsParameters.h"

// the package schould be capsulated in a name space
namespace marlintpc
{

    /** Helper Class which provides access to the parameters of ADC Electroncis.
     *  Its designed as interface for different ADC electronics conditions objects.
     *  It is derived from IConditionsChangeListener,
     *  The parameters are updated every time the conditions data change.
     * @author R. Diener (DESY)
     */

    class ADCElectronicsHandler : public lccd::IConditionsChangeListener
    {

        public:

            /// Map of channel pdestals.
            typedef std::map< int, tpcconddata::GenericADCElectronicsParameters* > GenericADCElectronicsParametersMap_t;

            /** Constructor
             * @param ColName name of the collection in which the ADC electronics parameters are stored
             */
            ADCElectronicsHandler( std::string ConditionsName, std::string ColName );

            /// Destructor
            virtual ~ADCElectronicsHandler( );

            static std::string getDefaultColName()
            {
                return  tpcconddata::GenericADCElectronicsParameters::getDefaultColName();
            };

            /// Updates the stored values if conditions have changed.
            /** Called by the Marlin ConditionsProcessor, if the condition change
             *  inplemetentation of the IConditionsChangeListener interface
             */
            void conditionsChanged( EVENT::LCCollection* col );

            /// get the polarity value, should be -1 or +1
            int getPolarity(int readoutGroupID);
            /// get the precision value of the electronics (maximum possible ADC value)
            int getMaxADCValue(int readoutGroupID);
            /// get the read out frequency of the read-out electronics (in MHz)
            float getFrequency(int readoutGroupID);

        private:
            std::string _conditionsName;
            char* _conditionsNameChar;
            tpcconddata::GenericADCElectronicsParameters* _genericADCElectronicsParameters;
            GenericADCElectronicsParametersMap_t _genericADCElectronicsParametersMap;

    };

}
#endif
