
// MarlinTPC
#include <ChannelCorrectionHandler.h>
#include "TPCCondDataExeptions.h"

//Marlin
#include <marlin/ConditionsProcessor.h>

// LCIO
#include <EVENT/LCGenericObject.h>

// stl
#include <algorithm>

using namespace std;
using namespace marlin;
using namespace tpcconddata;
using namespace EVENT;

namespace marlintpc {


ChannelCorrectionHandler::ChannelCorrectionHandler( string collection_name ) {

	// register this handler with the lccd conditions processor
	ConditionsProcessor::registerChangeListener( this, collection_name );
}


void ChannelCorrectionHandler::conditionsChanged( EVENT::LCCollection* col ) {

	// initialize the map
	_channel_corrections_by_channel.clear();

	const int entry_count = col->getNumberOfElements();

	for( int i = 0; i < entry_count; i++ ) {

		LCGenericObject *object = dynamic_cast<LCGenericObject*>( col->getElementAt( i ) );

		if( object == NULL ) {

			throw CondDataWrongObjectException( "The TPC ChannelCorrection collection contains invalid objects. Is the name incorrect?" );
		}

		ChannelCorrection channel_correction( object );

		pair<int, ChannelCorrection> new_element( channel_correction.getChannel(), channel_correction );
		pair<channel_map_t::iterator,bool> return_value;
		
		return_value = _channel_corrections_by_channel.insert( new_element );

		if( return_value.second == false ) {

			streamlog_out( WARNING ) << "The TPC ChannelCorrection collection contained two ChannelCorrection objects for a single channel ID. The o" << endl;
		}
	}
}


ChannelCorrection& ChannelCorrectionHandler::getChannelCorrection( int channelID ) {

	channel_map_t::iterator correction = _channel_corrections_by_channel.find( channelID );

	if( correction == _channel_corrections_by_channel.end() ) {

		streamlog_out( ERROR ) << "A channel correction was not available for channel " << channelID << '.' << endl;
		throw CondDataNotAvailableException( "A channel correction was not available for this channel." );
	}

	return (correction->second);
}


int ChannelCorrectionHandler::getChannelQuality( int channelID ) {
	
	return getChannelCorrection( channelID ).getChannelQuality();
}


int ChannelCorrectionHandler::getNChannelCalibrationFactors( int channelID ) {

	return getChannelCorrection( channelID ).getNChannelCalibrationFactors();
}


float ChannelCorrectionHandler::getChannelCalibrationFactor( int channelID, int index ) {
	
	return getChannelCorrection( channelID ).getChannelCalibrationFactor( index );
}


std::vector<float> ChannelCorrectionHandler::getChannelCalibrationFactors( int channelID ) {
	
	return getChannelCorrection( channelID ).getChannelCalibrationFactors();
}


float ChannelCorrectionHandler::getChannelTimeOffset( int channelID ) {
	
	return getChannelCorrection( channelID ).getChannelTimeOffset();
}


bool ChannelCorrectionHandler::isBroken( int channelID ) {
	
	return getChannelCorrection( channelID ).isBroken();
}


bool ChannelCorrectionHandler::isNoisy( int channelID ) {
	
	return getChannelCorrection( channelID ).isNoisy();
}


} // namespace marlintpc
