#include "TrackFitterSimpleMinimizer.h"
#include "TrackFitterFactory.h"
#include <gear/GEAR.h>
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>
#include <CLHEP/Vector/ThreeVector.h>

#include <streamlog/streamlog.h>

#include "IntersectionCalculator.h"
#include "LCObjectCopier.h"

#include <Minuit2/MnUserParameters.h>
#include <Minuit2/MnMigrad.h>
#include <Minuit2/FunctionMinimum.h>

#include "SimpleMinimizerResidualsFunction.h"

#include <streamlog/streamlog.h>

namespace marlintpc{

  TrackFitterBase * TrackFitterSimpleMinimizer::getInstance(LCParameters const *)
  {
    static TrackFitterSimpleMinimizer theSimpleFitter;
    return &theSimpleFitter;
  }

  TrackFitterSimpleMinimizer::TrackFitterSimpleMinimizer()
  {
    streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer:: constructor"<<std::endl;
  }

  std::string TrackFitterSimpleMinimizer::getRevision() const
  {
    return std::string( "$Rev:$" );
  }
  
  unsigned char TrackFitterSimpleMinimizer::getFitterType() const
  {
    return TrackFitterBase::FitterTypes::SIMPLEMINIMIZER;
  }

  IMPL::TrackImpl * TrackFitterSimpleMinimizer::fitTrack(EVENT::Track const * seedTrack) const
  {
    streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: copying track"<<std::endl;
   // create a copy of the track which is going to be fitted
    IMPL::TrackImpl * fittedTrack = LCObjectCopier::copy(seedTrack);

    streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: setting minuit parameters"<<std::endl;

    // prepare the starting parameters of the fit
    ROOT::Minuit2::MnUserParameters minuitParameters;
    minuitParameters.Add("D0",        seedTrack->getD0(), 0.1 );
    minuitParameters.Add("Phi",       seedTrack->getPhi(), 0.1 );
    minuitParameters.Add("TanLambda", seedTrack->getTanLambda(), 0.1 );
    minuitParameters.Add("Z0",        seedTrack->getZ0(), 0.1 );
    minuitParameters.Add("Omega",     seedTrack->getOmega(), 0.1 );
 
     streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: create theFCN"<<std::endl;

     SimpleMinimizerResidualsFunction theFCN(seedTrack, this);

     streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: creating migrad"<<std::endl;
   // create the minuit instance
    ROOT::Minuit2::MnMigrad migrad(theFCN,
				   minuitParameters);

     streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: performing fit"<<std::endl;
    
    // perform the minimisation
    ROOT::Minuit2::FunctionMinimum fitResult = migrad();

     streamlog_out(DEBUG2) << "TrackFitterSimpleMinimizer: setting track parameters"<<std::endl;

    // although in the documentation the << operator is not implemented. 
    // streamlog_out(DEBUG4) << fitResult << std::endl;

    fittedTrack->setD0(        fitResult.UserParameters().Value("D0") );
    fittedTrack->setPhi(       fitResult.UserParameters().Value("Phi") );
    fittedTrack->setTanLambda( fitResult.UserParameters().Value("TanLambda") );
    fittedTrack->setZ0(        fitResult.UserParameters().Value("Z0") );
    fittedTrack->setOmega(     fitResult.UserParameters().Value("Omega") );

    // finally set the fitter type in the trackType word
    setTrackFitterType( this->getFitterType(), fittedTrack );

    return fittedTrack;
  }

}//namespace marlintpc
