#ifndef GEARMAGNETICFIELD_H
#define GEARMAGNETICFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP

// MarlinTPC
#include "Field.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** GEARMagneticField : A magnetic field which is basically just a wrapper to accessing the global GEAR field. This is a much easier thing to use than putting a contant magnetic field into an LCCD object and accessing it through LCIO.
 *
 *    @author J. Abernathy (UVic)<br>
 */
class GEARMagneticField: public Field {

public:
	/** constructor
	 */
	GEARMagneticField();

	/** destructor
	 */
	virtual ~GEARMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

protected:

private:

};

} // namespace marlintpc

#endif // GEARMAGNETICFIELD_H
