
// stl
#include <string>

// marlintpc
#include "GlobalFieldProcessor.h"
#include "Field.h"

namespace marlintpc {

/** constructor
 */
Field::Field( std::string field_typename ):
	_field_error(CLHEP::Hep3Vector(0.,0.,0.)), _field_typename(field_typename) {

	// register this field with a global field manager or something like that
	GlobalFieldProcessor::get_instance().register_field( this );
}


/** destructor
 */
Field::~Field() {}


/** gets the field value at a specified point (in carthesian coordinates)
 */
CLHEP::HepVector Field::get_field_value( const CLHEP::HepVector & position ) {
	// convert to Hep3Vector and get the field value
	CLHEP::Hep3Vector field_vec = get_field_value( CLHEP::Hep3Vector(position[0], position[1], position[2]) );

	// convert the field value back into a HepVector
	CLHEP::HepVector field_value = CLHEP::HepVector(3);
	field_value[0] = field_vec.x();
	field_value[1] = field_vec.y();
	field_value[2] = field_vec.z();

	return field_value;
}

/** gets the field error. Assumes that the error is symmetric and independent of position
 */
CLHEP::Hep3Vector Field::get_field_error( ) {
	return _field_error;
}

/** returns the typename of the field
*/
std::string Field::get_typename() {

	return _field_typename;
}

} // namespace marlintpc
