#ifndef INTERSECTION_CALCULATOR_H
#define INTERSECTION_CALCULATOR_H

#include <exception>

namespace marlintpc
{

class IntersectionCalculator
{
 public:
    static std::pair<double,double> circle_circle(double R, double centreX, double centreY, double r);
    static std::pair<std::pair<double,double>,std::pair<double,double> >  circle_straightLine_xy(double R, double inclination, double xAxisIntersept);
    static std::pair<double,double>  circle_straightLine_phi(double R, double inclination, double xAxisIntersept);
    static std::pair<double,double>  horizonalLine_circle(double  Y,  double centreX, double centreY, double r);
    static double horizontalLine_straightLine(double  Y,  double inclination, double xAxisIntersept);

    // A nested exception which is thrown in case there is no intersection
    class NoIntersectionException : public std::exception 
    {
	protected:
	    std::string message ;
	    
	    NoIntersectionException(){  /*no_op*/ ; } 
	    
	public: 
	    virtual ~NoIntersectionException() throw() { /*no_op*/; } 
	    
	    NoIntersectionException( const std::string& text ){
		message = "NoIntersectionExceptionException: " + text ;
	    }
 
	    virtual const char* what() const  throw() { return  message.c_str() ; } 
    };

 private:
    // no need to generate an instance, all member functions are static
    IntersectionCalculator(){}
};

} //namespace marlintpc
#endif //INTERSECTION_CALCULATOR_H
