#include "RelativePosition.h"

#include <string>

#include "CLHEP/Vector/ThreeVector.h"

using namespace std;
using namespace CLHEP;

namespace marlintpc {

RelativePosition::RelativePosition() {

	_createdObject = true;

	set_x_position(0.);
	set_y_position(0.);
	set_z_position(0.);
	set_x_rotation(0.);
	set_y_rotation(0.);
}

RelativePosition::RelativePosition(EVENT::LCObject* obj) {

	_createdObject = false; // we did not create the object, so we must not delete it in the d'tor

	// initialise the object pointer
	_obj = dynamic_cast<IMPL::LCGenericObjectImpl*>(obj);

	if (_obj == 0) { // the cast did not succeed
		// could be an instance of LCFixedObject !?
		LCFixedObject<0,0,5>* f = dynamic_cast< LCFixedObject<0,0,5>* >( obj );

		if( f != 0 ) _obj = f->obj();
	}

	if (_obj == 0) {
		throw lcio::Exception("Cannot create RelativePosition from sth. that is not LCGenericObjectImpl" ) ;
	}

	// check the size of the object
	if ( 	( _obj->getNInt()		!= 0 )  ||
			( _obj->getNFloat()		!= 0 )  ||
			( _obj->getNDouble()	!= 5 )	) {
		throw lcio::Exception("RelativePosition(LCObject* obj): Wrong number of elements in object" ) ;
	}
}

RelativePosition::~RelativePosition() {

}

double RelativePosition::get_x_position( void ) const {
	return getDoubleVal( 0 );
}

double RelativePosition::get_y_position( void ) const {
	return getDoubleVal( 1 );
}

double RelativePosition::get_z_position( void ) const {
	return getDoubleVal( 2 );
}

double RelativePosition::get_x_rotation( void ) const {
	return getDoubleVal( 3 );
}

double RelativePosition::get_y_rotation( void ) const {
	return getDoubleVal( 4 );
}

Hep3Vector RelativePosition::get_position( void ) const {
	return Hep3Vector( get_x_position(), get_y_position(), get_z_position() );
}

void RelativePosition::set_x_position( double Value ) {
	_obj->setDoubleVal( 0, Value );
}

void RelativePosition::set_y_position( double Value ) {
	_obj->setDoubleVal( 1, Value );
}

void RelativePosition::set_z_position( double Value ) {
	_obj->setDoubleVal( 2, Value );
}

void RelativePosition::set_x_rotation( double Value ) {
	_obj->setDoubleVal( 3, Value );
}

void RelativePosition::set_y_rotation( double Value ) {
	_obj->setDoubleVal( 4, Value );
}

void RelativePosition::set_position( Hep3Vector position ) {
	set_x_position( position.x() );
	set_y_position( position.y() );
	set_z_position( position.z() );
}

const string RelativePosition::getTypeName( void ) const {

	return string("RelativePosition");
}

const string RelativePosition::getDataDescription( void ) const {

	return string("d:x_position,d:y_position,d:z_position,d:x_rotation,d:y_rotation");
}

}
