#ifndef LOAD_DATA_FILE_PROCESSOR_H
#define LOAD_DATA_FILE_PROCESSOR_H 1

//LCIO
#include <lcio.h>
#include <IO/LCReader.h>

//MARLIN
#include "marlin/Processor.h"

namespace marlintpc
{

    /** reads in an additional lcio file and adds all available collections 
     *  to the event with the same event number
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li Path to lcio file
     *
     *  <h4>Output</h4>
     *  \li all collections contained in that lcio for the given event
     *
     *  @param FilePath Path of the file to be read in
     *
     *
     *  @author Simone Zimmermann, University of Bonn
     */

    class LoadDataFileProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new LoadDataFileProcessor ; }
    
	LoadDataFileProcessor();

	~LoadDataFileProcessor();

	virtual void init();

	virtual void processRunHeader( EVENT::LCRunHeader* run ) ;

	virtual void processEvent( LCEvent * evt ) ; 
	
	virtual void end();
    
    protected:

	std::string _filePath; ///< Path to lcio file
	
	LCReader* _lcReader; ///< LCReader
	EVENT::LCRunHeader* _run; ///< run number
    };
}

#endif 
