#define LP1
#define DEBUG

#include "lcio.h"

#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"

#include "IMPL/LCEventImpl.h" 
#include "IMPL/LCRunHeaderImpl.h" 
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"
#include "IMPL/TrackerRawDataImpl.h"
#include "IMPL/LCFlagImpl.h"

#ifdef LP1
#include "gearimpl/VersatileDiskRowLayout.h"
#else
#include "gearimpl/RectangularPadRowLayout.h"
#endif

#include "gearimpl/Util.h"
#include "gearxml/GearXML.h"
#include "gear/GearMgr.h"
#include "gear/GEAR.h"

// global constants from Marlin, used for the global pointer to the GearMgr
#include <marlin/Global.h>

#include "ADCChannelMapping.h"
#include "Pedestal.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <CLHEP/Random/RandGauss.h>

#include "TRandom1.h"

#include "TKalDetCradle.h"
#include "TKalTrackState.h"
#include "TKalTrackSite.h"
#include "TVTrackHit.h"
#include "EXTPCKalDetector.h"
#include "EXTPCHit.h"
#include "EXEventGen.h"
#include "TObjArray.h"

using namespace std ;
using namespace lcio ;
using namespace gear;
using namespace tpcconddata;

void printUsage()
{
  cout << "generateHelix gearfile.xml [outfile.slcio]" <<endl;
    cout <<" outfile name is optional, "
	 << "default is TPCRawData.slcio"<< endl;
}

double Qfun(double x,            // Xtrack from central pad
            int    ipad=0,       // relative pad number (ipad=0: central pad)
            double wpitch = 1.,  // pad pitch
            double wq=0.5)       // charge width
{
  double xl = wpitch*(ipad - 0.5) - x;
  double xr = wpitch*(ipad + 0.5) - x;
  return 0.5*(TMath::Erf(xr/wq)-TMath::Erf(xl/wq));
}

// return true if hit is on pad plane, false otherwise
bool placeHit(float x, float y, float z, TPCParameters const &TPCParams, LCCollectionVec *trackerRawDataVec)
{

	  if (!TPCParams.isInsideModule(x,y))
          {
	    //cout << "Point " << x << " " << y << " is not in any module!" << endl; 
            return false;
          }
	  const TPCModule &module=TPCParams.getNearestModule(x,y);
#if 0
          if (module.getModuleID() != 3) return false;
#endif
          static const PadRowLayout2D& tpcPadLayout = module.getLocalPadLayout();
#ifdef LP1
	  static const VersatileDiskRowLayout  *pLayout = dynamic_cast< const VersatileDiskRowLayout*>(&tpcPadLayout);
#else
	  static const RectangularPadRowLayout *pLayout = dynamic_cast< const RectangularPadRowLayout*>(&tpcPadLayout);
#endif
	  int padNr;    // the pseudo hardware channel number, the same with padIndex:trival
          int padIndex; // the gear pad index

	  try
	  {
	    padIndex = module.getNearestPad(x,y);
	    padNr    = padIndex;
	  }
	  catch(exception& e)
	  {
	    // pad not on the pad plane
	    // break the loop
	    //cout << "Track left pad plane" << endl;
	    return false;
	  }

	  static int lasthitrow = -99999;
	  static int lasthitmod = -99999;
	  float      padPitch;

          if (lasthitrow !=module.getRowNumber(padIndex))
          {
            lasthitrow=module.getRowNumber(padIndex);
            lasthitmod=module.getModuleID();
#ifdef LP1
            padPitch = pLayout->rows().at(lasthitrow).getPadPitch_mm();
#else
            padPitch = pLayout->getPadPitch(padIndex);
#endif
          }
          else
	  {
            return false;
          }
            
          Vector2D padCenter= module.getPadCenter(padIndex);

          TrackerRawDataImpl* rawData = new TrackerRawDataImpl; 
          rawData->setCellID0(padNr);
          rawData->setCellID1(module.getModuleID());
#ifdef LP1
#if 1
	  float charge=(255. *(1.- fabs(y-padCenter[1])/padPitch) );
#else
	  float charge=255.*Qfun(y-padCenter[1], 0, padPitch);
#endif
#else
#if 1
	  float charge=(255. *(1.- fabs(x-padCenter[0])/padPitch) );
#else
	  float charge=255.*Qfun(x-padCenter[0], 0, padPitch);
#endif
#endif
	  EVENT::ShortVec ADCValues;

	  double freq   = module.getReadoutFrequency();
	  double vdrift = TPCParams.getDriftVelocity() ? TPCParams.getDriftVelocity() : 45000000.;
	  //cout << " freq = " << freq << " vdrift = " << vdrift << endl;
	  double tee   = z*freq/vdrift;
	  int    tslot = static_cast<int>(tee);
#if 0
	  ADCValues.push_back(static_cast<short>(charge*0.1));
	  ADCValues.push_back(static_cast<short>(charge*0.7));
	  ADCValues.push_back(static_cast<short>(charge*0.2));
#else
	  ADCValues.push_back(static_cast<short>(charge*Qfun(tee-tslot,-1,1.,1.)));
	  ADCValues.push_back(static_cast<short>(charge*Qfun(tee-tslot, 0,1.,1.)));
	  ADCValues.push_back(static_cast<short>(charge*Qfun(tee-tslot,+1,1.,1.)));
#endif
	  rawData->setADCValues(ADCValues);
	  rawData->setTime(tslot);

#ifdef DEBUG
	  // debug:
	  cout << "padIndex: " << padIndex 
               << "\t modle: " << module.getModuleID()
               << "\t row: " << module.getRowNumber(padIndex)
	       << "\t x/y: "<< x<<"/"<< y
	       << "\t center: "<< padCenter[0]<<"/"
	       << padCenter[1] 
	       << "\t charge: "<< charge
	       << endl;
#endif

	  // add the  rawData to the collection
	  trackerRawDataVec->push_back(rawData);

	   // in case of a central hit these is almost  no charge 
	  // on the neighbouring pad, skip it
#if 1
	  if (charge>253.)
	    return true;
#endif
	  // try to find neighbour which also gets charge
	  int neighbouringPadNr;
	  int neighbouringPadIndex;
	  try
	  {
#ifdef LP1
	    if (y-padCenter[1] < 0)
#else
	    if (x-padCenter[0] < 0)
#endif
	    {
	      neighbouringPadIndex = module.getRightNeighbour(padIndex);
	    }
	    else
	    {
	      neighbouringPadIndex = module.getLeftNeighbour(padIndex);
	    }
	    neighbouringPadNr = neighbouringPadIndex;
	  }
	  catch(exception& e)
	  {
	    //no neighbour found, must be edge pad
	    return true;
	  }

	  // if there is a neighbour calculate charge and add it to collection
	  padCenter = module.getPadCenter(neighbouringPadIndex);
	  
	  rawData = new TrackerRawDataImpl; 
	  rawData->setCellID0(neighbouringPadNr);
          rawData->setCellID1(module.getModuleID());
#ifdef LP1
#if 1
	  charge=(255. *(1.- fabs(y-padCenter[1])/padPitch) );
#else
	  charge=255.*Qfun(y-padCenter[1], 0, padPitch);
#endif
#else
#if 1
	  charge=(255. *(1.- fabs(x-padCenter[0])/padPitch) );
#else
	  charge=255.*Qfun(x-padCenter[0], 0, padPitch);
#endif
#endif
	  EVENT::ShortVec ADCValues2;
#if 0
	  ADCValues2.push_back(static_cast<short>(charge*0.1));
	  ADCValues2.push_back(static_cast<short>(charge*0.7));
	  ADCValues2.push_back(static_cast<short>(charge*0.2));
#else
	  ADCValues2.push_back(static_cast<short>(charge*Qfun(tee-tslot,-1,1.,1.)));
	  ADCValues2.push_back(static_cast<short>(charge*Qfun(tee-tslot, 0,1.,1.)));
	  ADCValues2.push_back(static_cast<short>(charge*Qfun(tee-tslot,+1,1.,1.)));
#endif
	  rawData->setADCValues(ADCValues2);
	  rawData->setTime(tslot);

#ifdef DEBUG
	  // debug:
	  cout << "   nb: " << neighbouringPadNr
	       << "\t x/y: "<< x<<"/"<<y
	       << "\t center: "<< padCenter[0]<<"/"
	       << padCenter[1] 
	       << "\t charge: "<< charge
	       << "\t z: " << z << " = "<<rawData->getTime()
	       << endl;
#endif

	  // add the  rawData to the collection
	  trackerRawDataVec->push_back(rawData);

	  return true;
}

int main(int argc, char **argv)
{

  if (argc!=2 && argc!=3 )
  {
    printUsage();
    return(1);
  }
  
  // read pad layout from gear
  GearXML gearXML( argv[1] ) ;
  GearMgr* gearMgr = gearXML.createGearMgr() ;
  marlin::Global::GEAR = gearMgr;
  TPCParameters const &TPCParams=gearMgr->getTPCParameters();
 
  // get outfile name
  string outFileName;
  if (argc==3)
    outFileName=argv[2];
  else
    outFileName="TPCRawData.slcio";

  // open outfile			     
  try {
    // create sio writer
    LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;
    
    try {
      lcWrt->open( outFileName, LCIO::WRITE_NEW ) ;
    }
    catch (IOException& e) {    
      cout << e.what() << endl ;
      return 0 ;
    }
      
    // create runheader and write it
    // the information should come from a file
    // or be given interactively in a future version
    LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ; 
    int runNr=0;
    runHdr->setRunNumber( runNr) ;
      
    string detName("lp1")  ;
    runHdr->setDetectorName( detName ) ;
      
    stringstream description ; 
    description << "lp1 rawData along straight line";
    runHdr->setDescription( description.str()  ) ;
      
    string tpcName("lp1") ;
    runHdr->addActiveSubdetector( tpcName ) ;
      
    lcWrt->writeRunHeader( runHdr ) ;
  

    //TKalDetCradle    lp1; // LP1 detector
    EXTPCKalDetector &tpcdet=*EXTPCKalDetector::GetInstance(); // TPC (tpc)
    TKalDetCradle    &lp1= const_cast<TKalDetCradle &>(dynamic_cast<const TKalDetCradle &>(tpcdet.GetParent()));
#if 0
    lp1.Install(tpcdet);  // install tpc into its toygld
    lp1.Close();          // close the cradle
    lp1.Sort();           // sort meas. layers from inside to outside
#endif
    TObjArray kalhits;                // array to store hits
    EXEventGen gen(lp1, kalhits);  // create event generator

    /////////FIXME//////////////////
    //double pt=-0.1;
    double pt=-5.;
    //double pt=-0.3;
    double cosmin=-0.05;
    double cosmax=+0.05;
    double phimin=-TMath::Pi()/2-0.01;
    double phimax=-TMath::Pi()/2+0.01;
    /////////FIXME//////////////////
     
    // loop the events, generate 11 events
    //int maxEvents=1;
    //int maxEvents=10;
    int maxEvents=1000;
    for (int eventNr=0; eventNr<maxEvents; eventNr++)
    {
      // create new lcio event
      LCEventImpl*  evt = new LCEventImpl() ;

      evt->setRunNumber(  runNr   ) ;
      evt->setEventNumber( eventNr ) ;
      evt->setDetectorName( detName ) ;
      
      // we want to store a collection of trackerRawData
      LCCollectionVec* trackerRawDataVec = new LCCollectionVec( LCIO::TRACKERRAWDATA )  ;
      LCFlagImpl flag(0);
      flag.setBit( LCIO:: TRAWBIT_ID1);
      trackerRawDataVec->setFlag( flag.getFlag() );

      kalhits.Delete(); // clear hit data
#if 0
	for (int ntk=0; ntk<4; ntk++) {
        THelicalTrack hel = gen.GenerateHelix(pt, cosmin, cosmax, phimin, phimax);
        gen.Swim(hel);
#else
#if 0
      THelicalTrack hel = gen.GenerateHelix(pt, cosmin, cosmin, 
               	                            phimin, phimin,
			                    //TVector3(0.0,-5.,15.));
			                    TVector3(0.,0.,15.));
      gen.Swim(hel);
#else
      THelicalTrack hel = gen.GenerateHelix(pt, cosmin, cosmax, 
               	                            phimin, phimax,
			                    //TVector3(0.0,-5.,15.));
			                    TVector3(0.,0.,15.));
      gen.Swim(hel);
#endif
#if 0
      THelicalTrack hel2 = gen.GenerateHelix(-pt, cosmax, cosmax, 
	                                     phimax+TMath::Pi(), phimax+TMath::Pi(),
				             TVector3(0.5,-5., 5.));
      gen.Swim(hel2);
#if 1
      THelicalTrack hel3 = gen.GenerateHelix(-pt, cosmax-0.1, cosmax-0.1, 
			                     phimax+TMath::Pi()+0.01, phimax+TMath::Pi()+0.01,
				             TVector3(0.6,-5., 10.));
					     //TVector3(0.6,-5., 7.));
      gen.Swim(hel3);
#if 1
      THelicalTrack hel4 = gen.GenerateHelix(pt, cosmin-0.1, cosmin-0.1, 
			                          phimin, phimin,
			                     TVector3(0.0,-5.,13.));
      gen.Swim(hel4);
#endif
#endif
#endif
#endif

#if 0
      int nhit = 0;
#endif
      TVTrackHit * thp=0;
      TIter next(&kalhits); 
      while ((thp=static_cast<TVTrackHit *>(next())))
      { 
        #ifdef OLD_UNIT_SYSTEM
         static const double kcm2mm=10.;
        #else
         static const double kcm2mm=1.;
        #endif
        const EXTPCMeasLayer & ml = static_cast<const EXTPCMeasLayer &>(thp->GetMeasLayer());
        TVector3 xv=kcm2mm*ml.HitToXv(*thp);
        placeHit(xv.X(), xv.Y(), xv.Z(), TPCParams, trackerRawDataVec);
#if 0
        cerr << ++nhit 
             << " (" << xv.X() << ","
                     << xv.Y() << ","
                     << xv.Z() << ")" << endl;
#endif
      }
#if 0
      }
#endif
      // add the rawData collection to the event
      evt->addCollection( trackerRawDataVec , "TPCRawData" ) ;

      // in the first event: write the mapping and the pedestals
      if (eventNr==0)
      {
        // create a collections of generic objects
        LCCollectionVec* mappingCol  = new LCCollectionVec( LCIO::LCGENERICOBJECT );
        LCCollectionVec* pedestalCol = new LCCollectionVec( LCIO::LCGENERICOBJECT );
	  
	//loop all pads and create a conditions objects for it

	// loop over modules
	int nmodule=TPCParams.getNModules();

        for (int m=0; m<nmodule; m++)
        {
	  const TPCModule &module = *TPCParams.getModules().at(m);

	  //outer loop: the rows
	  for (int row = 0; row < module.getNRows(); row++)
	  {
	    // vector of the pad indices in this row
	    std::vector<int> padsInRowVector = module.getPadsInRow(row);
	    
	    // nnow loop all pads in row
	    for (vector<int>::iterator padIndex=padsInRowVector.begin();
		 padIndex < padsInRowVector.end(); padIndex++)
	    {
	      int padNr = *padIndex;
	      // cout << "PadIndex=" << padNr << endl;
#if 0
	      mappingCol->push_back( new ADCChannelMapping(padNr, *padIndex ) );
#else
              mappingCol->push_back( new ADCChannelMapping(padNr, *padIndex , -1, module.getModuleID() , module.getModuleID()) );
#endif
	      // create pedestal with 0 as value and width, data are already zero-suppressed
              pedestalCol->push_back( new Pedestal(std::make_pair(padNr,0))) ; 
	    }
	  }
	}
	  // add the pe collection to the event
	  evt->addCollection( mappingCol , "TPCChannelMapping" ) ;
	  evt->addCollection( pedestalCol , "TPCPedestal" ) ;
      }

      // write the event to the file
      lcWrt->writeEvent( evt ) ;
	
      // dump the event to the screen 
      LCTOOLS::dumpEvent( evt ) ;

      // ------------ IMPORTANT ------------- !
      // we created the event so we need to delete it ...
      delete evt ;
      // -------------------------------------

    } // for eventNr

    delete runHdr ;
      
    lcWrt->close() ;
      
  } catch (std::exception& ex) {
    
    cout << " an excpetion occured: " << endl ;
    cout << "  " << ex.what() << endl ;
    return 1 ;
  }

  return 0;
}
