#ifndef HISTOGRAMTRACKERDATAPROCESSOR_H
#define HISTOGRAMTRACKERDATAPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

// In case you want AIDA histograms
namespace AIDA
{
        class IHistogram1D;
        class IHistogram2D;
}

/**  Processor to histogram the charge sum, the pulse maximum 
 *   and the pulse length of TrackerData. In see correlation 
 *   2D histograms of SumQ and MaxQ vs. the pulse length are also created.
 *
 *   @param InputCollectionName Name of the input TrackerData collection, default: TPCData
 *   @param NBinsSumQ Number of bins in the SumQ histogram, default: 100
 *   @param MaxSumQ Maximum in the SumQ histogram, default: 5000
 *   @param NBinsMaxQ Number of bins in the MaxQ histogram, default: 1024
 *   @param NBinsLength Number of bins in the length histogram, default: 50
 * 
 * @author M. Killenberg, CERN
 *
 */

class HistogramTrackerDataProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new HistogramTrackerDataProcessor; }
  
  HistogramTrackerDataProcessor();
  
  virtual ~HistogramTrackerDataProcessor();

  virtual void init();
  
  virtual void processRunHeader(EVENT::LCRunHeader* run);
  
  virtual void processEvent(EVENT::LCEvent* evt);
  
  virtual void check(EVENT::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  AIDA::IHistogram1D * _maxQHistogram; ///< Maximum ADC value
  AIDA::IHistogram1D * _sumQHistogram; ///< Sum of ADC Values
  AIDA::IHistogram1D * _pulseLengthHistogram; ///< Number of ADC Values
  AIDA::IHistogram2D * _lengthVsSumQHistogram;
  AIDA::IHistogram2D * _lengthVsMaxQHistogram;
  int _nBinsSumQ,_nBinsMaxQ,_nBinsLength; ///< Number of bins in the histograms
  float _maxSumQ; ///< Maximum of the SumQ histo (the others have nBins as range)

};
#endif // HISTOGRAMTRACKERDATAPROCESSOR_H
