#include "HistogramSimTrackerHitChargeSpectrumProcessor.h"
#include <iostream>
#include <string>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/SimTrackerHit.h>
#include <Exceptions.h>

using namespace lcio;
using namespace marlin;
using namespace std;

// Use AIDA histograms
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h> // or IHistogram2D

namespace marlintpc
{

HistogramSimTrackerHitChargeSpectrumProcessor aHistogramSimTrackerHitChargeSpectrumProcessor;


HistogramSimTrackerHitChargeSpectrumProcessor::HistogramSimTrackerHitChargeSpectrumProcessor() : Processor("HistogramSimTrackerHitChargeSpectrumProcessor")
{
  // the processor description
  _description = "HistogramSimTrackerHitChargeSpectrumProcessor is intended to ...";
  
  /* register steering parameters: name, description, class-variable, default value

    for example:
  */

        registerInputCollection(    LCIO::SIMTRACKERHIT,
            "InputCollectionName" ,
            "Name of the input TrackerData collection, default: PrimaryElectrons"  ,
            _inputColName ,
            string("PrimaryElectrons")) ;

  /*
        registerOutputCollection(   LCIO::TRACKERDATA,
          "OutputCollectionName" ,
          string("Name of the output TrackerData collection")+
          string("only the data related to a pulse will be stored")+
          string(\, default: TPCDataInPulses),
          _outputCollectionName ,
          string("TPCDataInPulses")) ;

        // It's nive to have the default value in the comment
        registerProcessorParameter( "MinPulsesPerHit" ,
	       "Minimum number of pulses for a valid hit, default: 3",
	       _minPulsesPerHit,
	       int(3));

        // Optional parameters should only be used when set
        // in the steering file.
        // The default value is there only for technical reasons,
        // it is meaningless.
        //
        // If properly implemented the cut is not applied when the parameter is not set:
        //   if (parameterSet("ChageCut")) { //apply the cut }
        //
        registerOptionalParameter( "ChargeCut" ,
            "If the charge is below this value the data will be ignored",
            _chargeCut,
           float(17.8));
  */

  registerProcessorParameter( "HistoNBins" ,
			      "Number of bins in the histogram, default: 100",
			      _histoNBins,
			      int(1000));

  registerProcessorParameter( "HistoMin",
			      "Minimum of histogram, default: 0.",
			      _histoMin,
			      float(0.));

  registerProcessorParameter( "HistoMax" ,
			      "Maximum of histogram, default: 0.",
			      _histoMax,
			      float(250.));

}


void HistogramSimTrackerHitChargeSpectrumProcessor::init()
{ 
  m_out(DEBUG) << " init() called  " << endl;
  
  // you might want to list the used parameters by invoking
  printParameters();

_myAidaHistogram =  AIDAProcessor::histogramFactory(this)->
                    createHistogram1D( "chargePerHit",
                                       "Charge per primary cluster",
                                       _histoNBins, _histoMin , _histoMax);

}


void HistogramSimTrackerHitChargeSpectrumProcessor::processRunHeader(LCRunHeader* run)
{
  // do something for each run
} 


void HistogramSimTrackerHitChargeSpectrumProcessor::processEvent(LCEvent* evt)
{ 
  // do something for each event, usually access the input collection:

  // use an LCCollectionVec instead of LCCollection so we can use iterators later
  LCCollectionVec* inputColVec = 0;

  // do a try / catch in case there is an empty event
  try
  {
    inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection( _inputColName ) );
  }
  catch (DataNotAvailableException &e)
  {
    // no input data, just return and process the next event
    return;
  }

  // loop the data collection and do something 
  for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
       inputObjectIter < inputColVec->end() ;
       inputObjectIter++ )
  {
    // dynamic cast the input object (LCObject) to what it really is e.g.
    SimTrackerHit * mySimTrackerHit = dynamic_cast<SimTrackerHit *>( *inputObjectIter );

    // for objects derrived from LCGenericObject or LCFixedObject (like conditions data)
    // you have to use special constructor
    // TPCConditions myTPCConditions( dynamic_cast<LCGenericObject*>( *inputObjectIter ) );
    
    //now do something with the data ...
    _myAidaHistogram->fill( mySimTrackerHit->getdEdx()/2.6e-8 );
  }
}


void HistogramSimTrackerHitChargeSpectrumProcessor::check(LCEvent* evt)
{ 
  // check what is needed here 
}


void HistogramSimTrackerHitChargeSpectrumProcessor::end()
{ 
  // do something at the end of the processing, e.g. cleaning up
}
} // close namespace brackets
